@@ -92,7 +92,7 @@ impl SdkLoggerProvider {
         if result.iter().all(|r| r.is_ok()) {
             Ok(())
         } else {
-            Err(OTelSdkError::InternalFailure(format!("errs: {:?}", result)))
+            Err(OTelSdkError::InternalFailure(format!("errs: {result:?}")))
         }
     }
 
@@ -283,21 +283,24 @@ impl LoggerProviderBuilder {
 
 #[cfg(test)]
 mod tests {
+    #[cfg(feature = "trace")]
+    use crate::logs::TraceContext;
+    #[cfg(feature = "trace")]
+    use crate::trace::SdkTracerProvider;
     use crate::{
-        logs::{InMemoryLogExporter, LogBatch, SdkLogRecord, TraceContext},
+        logs::{InMemoryLogExporter, LogBatch, SdkLogRecord},
         resource::{
             SERVICE_NAME, TELEMETRY_SDK_LANGUAGE, TELEMETRY_SDK_NAME, TELEMETRY_SDK_VERSION,
         },
-        trace::SdkTracerProvider,
         Resource,
     };
 
     use super::*;
+    use opentelemetry::logs::{AnyValue, LogRecord as _, Logger, LoggerProvider};
+    #[cfg(feature = "trace")]
+    use opentelemetry::trace::TraceContextExt;
+    #[cfg(feature = "trace")]
     use opentelemetry::trace::{SpanId, TraceId, Tracer as _, TracerProvider};
-    use opentelemetry::{
-        logs::{AnyValue, LogRecord as _, Logger, LoggerProvider},
-        trace::TraceContextExt,
-    };
     use opentelemetry::{Key, KeyValue, Value};
     use std::fmt::{Debug, Formatter};
     use std::sync::atomic::AtomicU64;
@@ -591,6 +594,7 @@ mod tests {
     }
 
     #[test]
+    #[cfg(feature = "trace")]
     fn trace_context_test() {
         let exporter = InMemoryLogExporter::default();
 
@@ -607,8 +611,8 @@ mod tests {
         tracer.in_span("test-span", |cx| {
             let ambient_ctxt = cx.span().span_context().clone();
             let explicit_ctxt = TraceContext {
-                trace_id: TraceId::from_u128(13),
-                span_id: SpanId::from_u64(14),
+                trace_id: TraceId::from(13),
+                span_id: SpanId::from(14),
                 trace_flags: None,
             };
 
@@ -798,7 +802,7 @@ mod tests {
 
             // Explicitly shut down the logger provider
             let shutdown_result = logger_provider1.shutdown();
-            println!("---->Result: {:?}", shutdown_result);
+            println!("---->Result: {shutdown_result:?}");
             assert!(shutdown_result.is_ok());
 
             // Verify that shutdown was called exactly once
